/* 
Michael Collins September 2022
Merging code from https://www.instructables.com/Measure-the-speed-of-Nerf-darts/
With 7Seg Display firmware from https://github.com/sparkfun/Serial7SegmentDisplay/wiki/Customizing-the-Display

Serial 7 Segment Display Firmware - amended to operate solely as a stand-alone MCU with display; no I2C, SPI, UART or buffer for incoming data on those interfaces.
Used SevSeg V3.7 which differs from the instructions at above link. Installed via Library Manager not manually from github.  Copyright 2020 Dean Reading <deanreading@hotmail.com>

 https://www.sparkfun.com/search/results?term=serial+7+segment&what=products available in red, blue, white, green. Orange would be nice to match Nerf.
  
 Note from original version - not sure it still applies: To use the additional pins, PB6 and PB7, on the ATmega328 we have to add 
 some maps to the pins_arduino.h file. This allows Arduino to identify PB6 as 
 digital pin 22, and PB7 as digital pin 23. Because the Serial 7-Segment runs on 
 the ATmega328's internal oscillator, these two pins open up for our use.

 An immprovements list:
 Double click to convert fps to m/s and back
 */
 
#include <Bounce2.h> // debounce function

//IR photo-transistors front and rear of barrel are assigned an ADC input each. On the 7SegDisplay only A6 and A7 are readily available, button is on SDA (A4)
int LSFrontPin = A7;  //light sensor at the front of the barrel
int LSBackPin = A6;  //light sensor at the back of the barrel
int TriggerPin = A4; // Looking for a button - ADC4 is SDA, so if we don't fire up the i2c stack, it can be used
int LSFrontVal = 0; // LS = Light Sensor
int LSBackVal = 0;

//Threshold variables - this will make sense during calibration
int BaseLineFront = 0; 
int BaseLineBack = 0;
float TempBaseLineFront = 0;
float TempBaseLineBack = 0;
int TriggerFront = 0;
int TriggerBack = 0;

//Speed variables
int measuringBack = 0;
float length = 0.068; // distance between phototransistors in meters  UPDATE THIS VALUE FOR EACH BUILD - 68mm in this case is 0.22 feet - easily edited here to display fps vs m/s
unsigned long measureLoops = 0;
unsigned long heartBeat = 0;

float meanVal = 0;
float meanCount = 0;

//Bounce Tester = Bounce(TriggerPin, 10); // Bounce.h instance created to monitor a pushbutton with 10ms threshold.
Bounce Tester = Bounce(); // Bounce2.h instance

// **/Instructable

#include "SevSeg.h" //Library to control generic seven segment displays.
SevSeg sevseg; //Create an instance of the object


//  Github for Sevseg v3.7 does not mention colon or apostrophe: https://github.com/DeanIsMe/SevSeg But neither are required here. 
byte hardwareConfig = COMMON_ANODE; // See SevSeg README.md for options
byte numDigits = 4;
byte digitPins[] = {16,17,3,4}; //Digits 1-4
byte segmentPins[]={8, 14, 6, A1, 23, 7, 5, 22}; // segments A-E and Decimal
bool resistorsOnSegments = true; // 'false' means resistors are on digit pins
bool updateWithDelays = false; // Default 'false' is Recommended
bool leadingZeros = false; // Use 'true' if you'd like to keep the leading zeros
bool disableDecPoint = false; // Use 'true' if your decimal point doesn't exist or isn't connected. Then, you only need to specify 7 segmentPins[]

void setup()
{  
  Tester.attach(TriggerPin, INPUT_PULLUP); //reading docs for Bounce2.h there are many diffs with bounce.h
  Tester.interval(50); //interval in ms
  
  Serial.begin(9600);
   //Initialize the SevSeg library with all the pins needed for this type of display
  sevseg.begin(hardwareConfig, numDigits, digitPins, segmentPins, resistorsOnSegments, updateWithDelays, leadingZeros, disableDecPoint);
  sevseg.setBrightness(90);
  sevseg.setNumber(8008,0); //Fixed value, decimal at right-most position. Seems to be no option to turn off decimals without disabling altogether

//   **INSTRUCTABLE**
  pinMode(LSFrontPin, INPUT);
  pinMode(LSBackPin, INPUT);
  pinMode(TriggerPin, INPUT_PULLUP); // Should this be before the bounce attach?
  

  //create a base line - sum 100 readings from the IR Phototransistors and divide by 100, set the trigger at 80%. All done in 1sec.
  for (int ii=0; ii<100; ii++){
    TempBaseLineFront = TempBaseLineFront + analogRead(LSFrontPin);
    TempBaseLineBack = TempBaseLineBack + analogRead(LSBackPin);
    delay(10);
  }
  BaseLineFront = (int)(TempBaseLineFront / 100);
  TriggerFront = (int)(0.8*BaseLineFront);
  BaseLineBack = (int)(TempBaseLineBack / 100);
  TriggerBack = (int)(0.8*BaseLineBack);

  //Show that the initialisation is complete.
 sevseg.setChars("NERF"); 
 Serial.print("TriggerFront: "); Serial.println(TriggerFront);
 Serial.print("TriggerBack: "); Serial.println(TriggerBack);
  
  measuringBack = 1; //start measuring and wait for the darts
  }

void loop()
{
   
 //read the push button
  Tester.update();
  int TesterStatus = Tester.read();
  //unsigned long duration = Tester.duration();
  if (TesterStatus == LOW && Tester.currentDuration() < 2000){ // if the button is pressed for less than 2s, display average, over 2s resets values.

sevseg.setNumber(meanVal*100/meanCount,2); // Shifted two places and two decimal places displayed.
Serial.print("meanVal: "); Serial.println(meanVal); Serial.print("meanCount: "); Serial.println(meanCount);

  }
  else if (TesterStatus == LOW && Tester.currentDuration() > 2000){
    sevseg.setNumber(8008,0); Serial.println("reset"); 
    meanVal = 0;
    meanCount = 0;
  }
  
  if (measuringBack == 1){
    //Dart enters the barrel, take a snapshot
    LSBackVal = analogRead(LSBackPin);
    if (LSBackVal < TriggerBack) { measuringBack = 0; measureLoops = micros(); Serial.print("Start Trigger :"); Serial.println(TriggerBack); Serial.println(LSBackVal); }
  } 
  else
  {
    //Dart reaches the front, take a snapshot
    LSFrontVal = analogRead(LSFrontPin);
    if (LSFrontVal < TriggerFront){
      unsigned long stopTime=micros(); 
      float deltaTus = (float)(stopTime-measureLoops); //took me a while - Delta Time in microseconds
      deltaTus = deltaTus / 1.0e6;  //convert from microseconds to seconds - now the name doesn't make sense.
      float mSpeed = length / deltaTus;
      measuringBack = 1; 

sevseg.setNumber(mSpeed*10,1); //shifted one position to account for a decimal
      
      meanCount = meanCount + 1;
      meanVal = meanVal + mSpeed; // Averaged at display time if button held
      
      Serial.print("measureLoops: ");Serial.println(measureLoops);
      Serial.print("stopTime: ");Serial.println(stopTime);      
      Serial.print("deltaT seconds: ");Serial.println(deltaTus);
      Serial.print("mSpeed: ");Serial.println(mSpeed);
      Serial.print("meanVal: ");Serial.println(meanVal);
      Serial.print("meanCount: ");Serial.println(meanCount);
      
       delay(200); // Might have to remove the delay for sevseg.
    }
  }
  
  
  //If rear sensor triggered and front sensor not triggered after 1s, start again)
  if ((measuringBack == 0) && (micros() > (measureLoops + 1000000)))
  {
    measuringBack = 1;
  }
    
    sevseg.refreshDisplay(); // Must run repeatedly
    
}
